@@ -85,6 +85,9 @@ ControlWidget::ControlWidget(const QString &ruleFile, Control *parent)
     connect(ui->pbStart, SIGNAL(clicked()), this, SLOT(simAppStart()));
     connect(ui->pbAbort, SIGNAL(clicked()), this, SLOT(simAppAbort()));
     connect(ui->pbReset, SIGNAL(clicked()), this, SLOT(modemSilentReset()));
+    connect(ui->pbSendGNSSData, SIGNAL(clicked()), this, SLOT(sendGNSSData()));
+    connect(ui->pbGNSSDefault, SIGNAL(clicked()), this, SLOT(setDefaultGNSSData()));
+    connect(ui->pbSendNotif, SIGNAL(clicked()), this, SLOT(sendCSSN()));
 
     QStringList headers;
     headers << "Sender" << "Priority" << "Notification Status";
@@ -92,6 +95,7 @@ ControlWidget::ControlWidget(const QString &ruleFile, Control *parent)
     ui->twMessageList->verticalHeader()->hide();
 
     handleNewApp();
+    handleCSSNNotif();
 
     show();
 }
@@ -130,6 +134,64 @@ Control::~Control()
     delete widget;
 }
 
+void Control::callManagement( QList<CallInfo> *list )
+{
+    int row = 0;
+    bool enableCSSU = false;
+    bool enableCSSI = false;
+    static QString state_table[] = { tr("Active"), tr("Held"),
+                                      tr("Dialing"), tr("Alerting"),
+                                      tr("Incoming"), tr("Waiting"),
+                                      tr("Disconnected"), tr("Swapping") };
+
+    widget->clearCallView();
+
+    foreach( CallInfo i, *list ) {
+        QString param[5];
+
+        if ( i.incoming && !enableCSSU )
+            enableCSSU = true;
+        if ( !i.incoming && !enableCSSI )
+            enableCSSI = true;
+
+        param[0].setNum( i.id );
+        param[1] = i.number;
+        param[2] = state_table[i.state];
+        param[3] = i.name;
+        param[4] = i.incoming ? tr("Incoming") : tr("Outgoing");
+
+        widget->updateCallView( param, row );
+        row++;
+    }
+
+    widget->setCssiEnabled( enableCSSI );
+    widget->setCssuEnabled( enableCSSU );
+}
+
+void ControlWidget::setCssiEnabled( bool enableCSSI )
+{
+    ui->cbCSSI->setEnabled( enableCSSI );
+}
+
+void ControlWidget::setCssuEnabled( bool enableCSSU )
+{
+    ui->cbCSSU->setEnabled( enableCSSU );
+}
+
+void ControlWidget::clearCallView()
+{
+    ui->twCallMgt->clearContents();
+}
+
+void ControlWidget::updateCallView( QString callParameters [5], int row )
+{
+    if ( row > ui->twCallMgt->rowCount() - 1 )
+        ui->twCallMgt->insertRow( row );
+
+    for ( int i = 0; i < 5; i++ )
+        ui->twCallMgt->setItem( row, i, new QTableWidgetItem( callParameters[i] ) );
+}
+
 void Control::setPhoneNumber( const QString &number )
 {
     widget->setWindowTitle("Phonesim - Number: " + number);
@@ -140,13 +202,42 @@ void Control::warning( const QString &title, const QString &message )
     QMessageBox::warning(widget, title, message, "OK");
 }
 
+void ControlWidget::handleCSSNNotif()
+{
+    ui->cbCSSU->setEnabled( false );
+    ui->cbCSSI->setEnabled( false );
+
+    ui->cbCSSU->insertItem( 0, "" );
+    ui->cbCSSU->insertItem( 1, "0 - forwarded", 0 );
+    ui->cbCSSU->insertItem( 3, "2 - on hold", 2 );
+    ui->cbCSSU->insertItem( 4, "3 - retrieved", 3 );
+    ui->cbCSSU->insertItem( 5, "4 - multiparty", 4 );
+
+    ui->cbCSSI->insertItem( 0, "" );
+    ui->cbCSSI->insertItem( 3, "2 - forwarded", 2 );
+    ui->cbCSSI->insertItem( 6, "5 - outgoing barred", 5 );
+    ui->cbCSSI->insertItem( 7, "6 - incoming barred", 6 );
+}
+
+void ControlWidget::sendCSSN()
+{
+    QVariant v = ui->cbCSSU->itemData( ui->cbCSSU->currentIndex() );
+
+    if ( v.canConvert<int>() && ui->cbCSSU->isEnabled() )
+        emit unsolicitedCommand( "+CSSU: "+QString::number( v.toInt() ) );
+
+    v = ui->cbCSSI->itemData( ui->cbCSSI->currentIndex() );
+
+    if ( v.canConvert<int>() && ui->cbCSSI->isEnabled() )
+        emit unsolicitedCommand( "+CSSI: "+QString::number( v.toInt() ) );
+}
+
 void ControlWidget::sendSQ()
 {
     emit variableChanged("SQ",QString::number(ui->hsSignalQuality->value())+",99");
     emit unsolicitedCommand("+CSQ: "+QString::number(ui->hsSignalQuality->value())+",99");
 }
 
-
 void ControlWidget::sendBC()
 {
     bool charging = ui->hsBatteryCharging->checkState() == Qt::Checked;
@@ -222,6 +313,33 @@ void ControlWidget::sendSMSMessage()
     p->constructSMSMessage(ui->leSMSClass->text().toInt(), ui->leMessageSender->text(), ui->leSMSServiceCenter->text(), ui->teSMSText->toPlainText());
 }
 
+void ControlWidget::setDefaultGNSSData()
+{
+    ui->teGNSStext->clear();
+
+    ui->teGNSStext->append("<?xml version=\"1.0\" ?>");
+    ui->teGNSStext->append("<pos>");
+    ui->teGNSStext->append("  <pos_err>");
+    ui->teGNSStext->append("    <err_reason literal=\"not_enough_gps_satellites\" />");
+    ui->teGNSStext->append("  </pos_err>");
+    ui->teGNSStext->append("</pos>");
+}
+
+void ControlWidget::sendGNSSData()
+{
+    QStringList xml = ui->teGNSStext->toPlainText().split("\n");
+
+    foreach ( QString line, xml )
+    {
+        if (!line.isEmpty()) {
+            QString cposr =  "+CPOSR: " + line;
+
+            emit unsolicitedCommand(cposr);
+            handleFromData( cposr );
+        }
+    }
+}
+
 void ControlWidget::sendMGD()
 {
     emit command("AT+CMGD=1");
@@ -505,6 +623,9 @@ Script::Script(QObject *obj, Ui_ControlBase *ui) : QDBusAbstractAdaptor(obj)
 
     QScriptValue qsTab6 = engine.newQObject(ui->tab_6);
     engine.globalObject().setProperty("tabSIM", qsTab6);
+
+    QScriptValue qsTab8 = engine.newQObject(ui->tab_8);
+    engine.globalObject().setProperty("tabPosition", qsTab8);
 }
 
 void Script::SetPath(const QString &path, const QDBusMessage &msg)
