@@ -21,6 +21,7 @@
 #include <qatutils.h>
 #include <qdebug.h>
 #include <QTextCodec>
+#include "qsmsmessage.h"
 
 class SimApplicationPrivate
 {
@@ -258,7 +259,8 @@ void SimApplication::endSession()
 }
 
 DemoSimApplication::DemoSimApplication( SimRules *rules, QObject *parent )
-    : SimApplication( rules, parent )
+    : SimApplication( rules, parent ), smsDestNumber( "12345" ),
+    smsText( "Hello" )
 {
 }
 
@@ -284,6 +286,12 @@ const QString DemoSimApplication::getName()
 #define MainMenu_SendSS     11
 #define MainMenu_Language   12
 #define MainMenu_SendUSSD   13
+#define MainMenu_SendSMS    14
+#define MainMenu_Polling    15
+#define MainMenu_Timers     16
+#define MainMenu_Refresh    17
+#define MainMenu_LocalInfo  18
+#define MainMenu_BIP        19
 
 #define SportsMenu_Chess        1
 #define SportsMenu_Painting     2
@@ -301,6 +309,7 @@ const QString DemoSimApplication::getName()
 #define SendSSMenu_CLIR     5
 #define SendSSMenu_CoLP     6
 #define SendSSMenu_CoLR     7
+#define SendSSMenu_CNAP     8
 
 #define CBMenu_Activation       1
 #define CBMenu_Interrogation    2
@@ -316,21 +325,17 @@ const QString DemoSimApplication::getName()
 #define CWMenu_Interrogation    2
 #define CWMenu_Deactivation     3
 
-#define CLIPMenu_Activation       1
-#define CLIPMenu_Interrogation    2
-#define CLIPMenu_Deactivation     3
+#define CLIPMenu_Interrogation    1
 
 #define CLIRMenu_Activation       1
 #define CLIRMenu_Interrogation    2
 #define CLIRMenu_Deactivation     3
 
-#define CoLPMenu_Activation       1
-#define CoLPMenu_Interrogation    2
-#define CoLPMenu_Deactivation     3
+#define CoLPMenu_Interrogation    1
 
-#define CoLRMenu_Activation       1
-#define CoLRMenu_Interrogation    2
-#define CoLRMenu_Deactivation     3
+#define CoLRMenu_Interrogation    1
+
+#define CNAPMenu_Interrogation    1
 
 #define Language_Specific       1
 #define Language_Non_Specific   2
@@ -342,6 +347,34 @@ const QString DemoSimApplication::getName()
 #define SendUSSD_Error      4
 #define SendUSSD_Main       5
 
+#define LocalInfoMenu_Time  1
+#define LocalInfoMenu_Lang  2
+
+#define BIPMenu_OpenChannel      1
+#define BIPMenu_CloseChannel     2
+#define BIPMenu_ReceiveData      3
+#define BIPMenu_SendData         4
+#define BIPMenu_GetChannelStatus 5
+
+enum SendSMSMenuItems {
+	SendSMS_Unpacked = 1,
+	SendSMS_Packed,
+	SendSMS_SetDestination,
+	SendSMS_SetContents,
+};
+
+enum PollingMenuItems {
+	Polling_Off = 1,
+	Polling_30s,
+};
+
+enum TimersMenuItems {
+	Timers_Start = 1,
+	Timers_Stop,
+	Timers_Sleep,
+	Timers_Query,
+};
+
 void DemoSimApplication::mainMenu()
 {
     QSimCommand cmd;
@@ -402,6 +435,30 @@ void DemoSimApplication::mainMenu()
     item.setLabel( "Send USSD" );
     items += item;
 
+    item.setIdentifier( MainMenu_SendSMS );
+    item.setLabel( "Send SMS request" );
+    items += item;
+
+    item.setIdentifier( MainMenu_Polling );
+    item.setLabel( "SIM Polling" );
+    items += item;
+
+    item.setIdentifier( MainMenu_Timers );
+    item.setLabel( "Timers" );
+    items += item;
+
+    item.setIdentifier( MainMenu_Refresh );
+    item.setLabel( "SIM Refresh" );
+    items += item;
+
+    item.setIdentifier( MainMenu_LocalInfo );
+    item.setLabel( "Provide Local Information" );
+    items += item;
+
+    item.setIdentifier( MainMenu_BIP );
+    item.setLabel( "BIP commands" );
+    items += item;
+
     cmd.setMenuItems( items );
 
     command( cmd, 0, 0 );
@@ -514,6 +571,42 @@ void DemoSimApplication::mainMenuSelection( int id )
         }
         break;
 
+        case MainMenu_SendSMS:
+        {
+            sendSMSMenu();
+        }
+        break;
+
+        case MainMenu_Polling:
+        {
+            sendPollingMenu();
+        }
+        break;
+
+        case MainMenu_Timers:
+        {
+            sendTimersMenu();
+        }
+        break;
+
+        case MainMenu_Refresh:
+        {
+            sendRefreshMenu();
+        }
+        break;
+
+        case MainMenu_LocalInfo:
+        {
+            sendLocalInfoMenu();
+        }
+        break;
+
+        case MainMenu_BIP:
+        {
+            sendBIPMenu();
+        }
+        break;
+
         default:
         {
             // Don't know what this item is, so just re-display the main menu.
@@ -1075,7 +1168,11 @@ void DemoSimApplication::sendSendSSMenu()
     items += item;
 
     item.setIdentifier( SendSSMenu_CLIP );
-    item.setLabel( "CLIP (Calling Line Idnetification Presentation)" );
+    item.setLabel( "CLIP (Calling Line Identification Presentation)" );
+    items += item;
+
+    item.setIdentifier( SendSSMenu_CNAP );
+    item.setLabel( "CNAP (Calling Name Identification Presentation)" );
     items += item;
 
     item.setIdentifier( SendSSMenu_CLIR );
@@ -1401,18 +1498,10 @@ void DemoSimApplication::sendCLIPMenu()
     cmd.setType( QSimCommand::SelectItem );
     cmd.setTitle( "CLIP" );
 
-    item.setIdentifier( CLIPMenu_Activation );
-    item.setLabel( "Activation" );
-    items += item;
-
     item.setIdentifier( CLIPMenu_Interrogation );
     item.setLabel( "Interrogation" );
     items += item;
 
-    item.setIdentifier( CLIPMenu_Deactivation );
-    item.setLabel( "Deactivation" );
-    items += item;
-
     cmd.setMenuItems( items );
 
     command( cmd, this, SLOT(CLIPMenu(QSimTerminalResponse)) );
@@ -1424,33 +1513,14 @@ void DemoSimApplication::CLIPMenu( const QSimTerminalResponse& resp )
 
     if ( resp.result() == QSimTerminalResponse::Success ) {
         switch ( resp.menuItem() ) {
-
-            case CLIPMenu_Activation:
-            {
-                cmd.setType( QSimCommand::SendSS );
-                cmd.setDestinationDevice( QSimCommand::Network );
-                cmd.setNumber( "*30#" );
-                command( cmd, this, SLOT(sendCLIPMenu()) );
-            }
-            break;
-
             case CLIPMenu_Interrogation:
             {
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "*#30#" );
                 command( cmd, this, SLOT(sendCLIPMenu()) );
+                break;
             }
-            break;
-
-            case CLIPMenu_Deactivation:
-            {
-                cmd.setType( QSimCommand::SendSS );
-                cmd.setDestinationDevice( QSimCommand::Network );
-                cmd.setNumber( "#30#" );
-                command( cmd, this, SLOT(sendCLIPMenu()) );
-            }
-            break;
 
             default:
                 endSession();
@@ -1543,18 +1613,10 @@ void DemoSimApplication::sendCoLPMenu()
     cmd.setType( QSimCommand::SelectItem );
     cmd.setTitle( "CoLP" );
 
-    item.setIdentifier( CoLPMenu_Activation );
-    item.setLabel( "Activation" );
-    items += item;
-
     item.setIdentifier( CoLPMenu_Interrogation );
     item.setLabel( "Interrogation" );
     items += item;
 
-    item.setIdentifier( CoLPMenu_Deactivation );
-    item.setLabel( "Deactivation" );
-    items += item;
-
     cmd.setMenuItems( items );
 
     command( cmd, this, SLOT(CoLPMenu(QSimTerminalResponse)) );
@@ -1566,33 +1628,14 @@ void DemoSimApplication::CoLPMenu( const QSimTerminalResponse& resp )
 
     if ( resp.result() == QSimTerminalResponse::Success ) {
         switch ( resp.menuItem() ) {
-
-            case CoLPMenu_Activation:
-            {
-                cmd.setType( QSimCommand::SendSS );
-                cmd.setDestinationDevice( QSimCommand::Network );
-                cmd.setNumber( "*76#" );
-                command( cmd, this, SLOT(sendCoLPMenu()) );
-            }
-            break;
-
             case CoLPMenu_Interrogation:
             {
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "*#76#" );
                 command( cmd, this, SLOT(sendCoLPMenu()) );
+                break;
             }
-            break;
-
-            case CoLPMenu_Deactivation:
-            {
-                cmd.setType( QSimCommand::SendSS );
-                cmd.setDestinationDevice( QSimCommand::Network );
-                cmd.setNumber( "#76#" );
-                command( cmd, this, SLOT(sendCoLPMenu()) );
-            }
-            break;
 
             default:
                 endSession();
@@ -1614,18 +1657,10 @@ void DemoSimApplication::sendCoLRMenu()
     cmd.setType( QSimCommand::SelectItem );
     cmd.setTitle( "CoLR" );
 
-    item.setIdentifier( CoLRMenu_Activation );
-    item.setLabel( "Activation" );
-    items += item;
-
     item.setIdentifier( CoLRMenu_Interrogation );
     item.setLabel( "Interrogation" );
     items += item;
 
-    item.setIdentifier( CoLRMenu_Deactivation );
-    item.setLabel( "Deactivation" );
-    items += item;
-
     cmd.setMenuItems( items );
 
     command( cmd, this, SLOT(CoLRMenu(QSimTerminalResponse)) );
@@ -1637,33 +1672,58 @@ void DemoSimApplication::CoLRMenu( const QSimTerminalResponse& resp )
 
     if ( resp.result() == QSimTerminalResponse::Success ) {
         switch ( resp.menuItem() ) {
-
-            case CoLRMenu_Activation:
-            {
-                cmd.setType( QSimCommand::SendSS );
-                cmd.setDestinationDevice( QSimCommand::Network );
-                cmd.setNumber( "*77#" );
-                command( cmd, this, SLOT(sendCoLRMenu()) );
-            }
-            break;
-
             case CoLRMenu_Interrogation:
             {
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "*#77#" );
                 command( cmd, this, SLOT(sendCoLRMenu()) );
+                break;
             }
-            break;
 
-            case CoLRMenu_Deactivation:
+            default:
+                endSession();
+                break;
+        }
+    } else if ( resp.result() == QSimTerminalResponse::BackwardMove ) {
+        sendSendSSMenu();
+    } else {
+        endSession();
+    }
+}
+
+void DemoSimApplication::sendCNAPMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "CNAP" );
+
+    item.setIdentifier( CNAPMenu_Interrogation );
+    item.setLabel( "Interrogation" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(CNAPMenu(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::CNAPMenu( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() == QSimTerminalResponse::Success ) {
+        switch ( resp.menuItem() ) {
+            case CNAPMenu_Interrogation:
             {
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
-                cmd.setNumber( "#77#" );
-                command( cmd, this, SLOT(sendCoLRMenu()) );
+                cmd.setNumber( "*#300#" );
+                command( cmd, this, SLOT(sendCNAPMenu()) );
+                break;
             }
-            break;
 
             default:
                 endSession();
@@ -1832,3 +1892,623 @@ void DemoSimApplication::USSDMenu( const QSimTerminalResponse& resp )
         endSession();
     }
 }
+
+void DemoSimApplication::sendSMSMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "Send SMS" );
+
+    item.setIdentifier( SendSMS_Unpacked );
+    item.setLabel( "Unpacked" );
+    items += item;
+
+    item.setIdentifier( SendSMS_Packed );
+    item.setLabel( "Packed text" );
+    items += item;
+
+    item.setIdentifier( SendSMS_SetDestination );
+    item.setLabel( "Set destination (" + smsDestNumber + ")" );
+    items += item;
+
+    item.setIdentifier( SendSMS_SetContents );
+    item.setLabel( "Set content text" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(smsMenuResp(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::smsMenuResp( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() != QSimTerminalResponse::Success ) {
+        /* Unknown response - just go back to the main menu. */
+        endSession();
+
+        return;
+    }
+
+    /* Item selected. */
+    switch ( resp.menuItem() ) {
+    case SendSMS_Unpacked:
+        break;
+
+    case SendSMS_Packed:
+        cmd.setSmsPacking( true );
+        break;
+
+    case SendSMS_SetDestination:
+        cmd.setType( QSimCommand::GetInput );
+        cmd.setText( "Enter recipient number" );
+        cmd.setWantDigits( true );
+        cmd.setMinimumLength( 2 );
+        cmd.setMaximumLength( 20 );
+        cmd.setDefaultText( smsDestNumber );
+        command( cmd, this, SLOT(smsSetDestResp(QSimTerminalResponse)) );
+        return;
+
+    case SendSMS_SetContents:
+        cmd.setType( QSimCommand::GetInput );
+        cmd.setText( "Enter message text" );
+        cmd.setMaximumLength( 100 );
+        cmd.setDefaultText( smsText );
+        command( cmd, this, SLOT(smsSetTextResp(QSimTerminalResponse)) );
+        return;
+    }
+
+    QSMSMessage sms;
+    sms.setValidityPeriod( -1 );
+    sms.setMessageClass( 2 );
+    sms.setProtocol( 0 );
+    sms.setRecipient( smsDestNumber );
+    sms.setText( smsText );
+    sms.setForceGsm( false );
+    sms.setBestScheme( QSMS_8BitAlphabet );
+    sms.setDataCodingScheme( 0xf6 );
+
+    cmd.setType( QSimCommand::SendSMS );
+    cmd.setText( "Sending an SMS to our friends at " + smsDestNumber );
+    cmd.setNumber( "123" );
+    cmd.addExtensionField( 0x8b, sms.toPdu().mid( 1 ) );
+    cmd.setDestinationDevice( QSimCommand::Network );
+
+    command( cmd, this, SLOT(endSession()) );
+}
+
+void DemoSimApplication::smsSetDestResp( const QSimTerminalResponse& resp )
+{
+    if ( resp.result() == QSimTerminalResponse::BackwardMove ) {
+        sendSMSMenu();
+
+        return;
+    }
+
+    if ( resp.result() != QSimTerminalResponse::Success ) {
+        /* Unknown response - just go back to the main menu. */
+        endSession();
+
+        return;
+    }
+
+    smsDestNumber = resp.text();
+    sendSMSMenu();
+}
+
+void DemoSimApplication::smsSetTextResp( const QSimTerminalResponse& resp )
+{
+    if ( resp.result() == QSimTerminalResponse::BackwardMove ) {
+        sendSMSMenu();
+
+        return;
+    }
+
+    if ( resp.result() != QSimTerminalResponse::Success ) {
+        /* Unknown response - just go back to the main menu. */
+        endSession();
+
+        return;
+    }
+
+    smsText = resp.text();
+    sendSMSMenu();
+}
+
+void DemoSimApplication::sendPollingMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "Polling Menu" );
+
+    item.setIdentifier( Polling_Off );
+    item.setLabel( "Polling Off" );
+    items += item;
+
+    item.setIdentifier( Polling_30s );
+    item.setLabel( "Poll Interval of 30s" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(pollingMenuResp(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::pollingMenuResp( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() != QSimTerminalResponse::Success ) {
+        /* Unknown response - just go back to the main menu. */
+        endSession();
+
+        return;
+    }
+
+    /* Item selected. */
+    switch ( resp.menuItem() ) {
+    case Polling_Off:
+        cmd.setType( QSimCommand::PollingOff );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        command( cmd, this, SLOT(endSession()) );
+        break;
+
+    case Polling_30s:
+        cmd.setType( QSimCommand::PollInterval );
+        cmd.setDuration( 30000 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        command( cmd, this, SLOT(endSession()) );
+        break;
+    }
+}
+
+void DemoSimApplication::sendTimersMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "Timer ops" );
+
+    item.setIdentifier( Timers_Start );
+    item.setLabel( "Reset timer 1 to 1h" );
+    items += item;
+
+    item.setIdentifier( Timers_Stop );
+    item.setLabel( "Stop all timers" );
+    items += item;
+
+    item.setIdentifier( Timers_Sleep );
+    item.setLabel( "Sleep for 10s using timer 2" );
+    items += item;
+
+    item.setIdentifier( Timers_Query );
+    item.setLabel( "Show statuses" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(timersMenuResp(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::timersMenuResp( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() != QSimTerminalResponse::Success ) {
+        /* Unknown response - just go back to the main menu. */
+        endSession();
+
+        return;
+    }
+
+    /* Item selected. */
+    switch ( resp.menuItem() ) {
+    case Timers_Start:
+        cmd.setQualifier( 0 );
+        cmd.setType( QSimCommand::TimerManagement );
+        cmd.setTimerId( 1 );
+        cmd.setDuration( 3600000 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        command( cmd, this, SLOT(endSession()) );
+        break;
+
+    case Timers_Stop:
+        cmd.setQualifier( 1 );
+        cmd.setType( QSimCommand::TimerManagement );
+        cmd.setTimerId( 1 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        command( cmd, this, SLOT(timersCmdResp(QSimTerminalResponse)) );
+        break;
+
+    case Timers_Sleep:
+        cmd.setQualifier( 0 );
+        cmd.setType( QSimCommand::TimerManagement );
+        cmd.setTimerId( 2 );
+        cmd.setDuration( 10000 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        command( cmd, NULL, NULL );
+        break;
+
+    case Timers_Query:
+        cmd.setQualifier( 2 );
+        cmd.setType( QSimCommand::TimerManagement );
+        cmd.setTimerId( 1 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        command( cmd, this, SLOT(timersCmdResp(QSimTerminalResponse)) );
+	timerStatus = "";
+        break;
+    }
+}
+
+void DemoSimApplication::timersCmdResp( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.command().qualifier() == 1 ) {
+        if ( resp.command().timerId() < 1 ) {
+            endSession();
+            return;
+        }
+
+        /* Stop the next timer */
+        cmd.setQualifier( 1 );
+        cmd.setType( QSimCommand::TimerManagement );
+        cmd.setTimerId( resp.timerId() + 1 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+
+        if ( cmd.timerId() >= 8 )
+            command( cmd, this, SLOT(endSession()) );
+        else
+            command( cmd, this, SLOT(timersCmdResp(QSimTerminalResponse)) );
+
+        return;
+    }
+
+    if ( resp.command().qualifier() == 2 ) {
+        if ( resp.result() == QSimTerminalResponse::Success ) {
+            QString status;
+            status.sprintf( "Timer %i expires in %i:%02i:%02i.\n",
+                    resp.timerId(), resp.duration() / 3600000,
+                    (resp.duration() / 60000) % 60,
+                    (resp.duration() / 1000) % 60 );
+
+            timerStatus += status;
+        }
+
+        if ( resp.timerId() == 8 || resp.timerId() < 1 ) {
+            /* All timers done */
+            if ( timerStatus.length() == 0 )
+                timerStatus = "All timers are stopped.";
+
+            cmd.setType( QSimCommand::DisplayText );
+            cmd.setDestinationDevice( QSimCommand::Display );
+            cmd.setText( timerStatus.left(220) );
+            command( cmd, this, SLOT(endSession()) );
+            return;
+        }
+
+        /* Interrogate the next timer */
+        cmd.setQualifier( 2 );
+        cmd.setType( QSimCommand::TimerManagement );
+        cmd.setTimerId( resp.timerId() + 1 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        command( cmd, this, SLOT(timersCmdResp(QSimTerminalResponse)) );
+
+        return;
+    }
+}
+
+bool DemoSimApplication::envelope( const QSimEnvelope& env )
+{
+    if ( env.type() != QSimEnvelope::TimerExpiration )
+        return SimApplication::envelope( env );
+
+    QSimCommand cmd;
+
+    cmd.setType( QSimCommand::DisplayText );
+    cmd.setDestinationDevice( QSimCommand::Display );
+    cmd.setText( "Timer expired." );
+    command( cmd, this, SLOT(endSession()) );
+
+    return true;
+}
+
+void DemoSimApplication::sendRefreshMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "SIM Refresh menu" );
+
+    /* Use qualifier value + 1 for id */
+    item.setIdentifier( 1 );
+    item.setLabel( "NAA Initialization+Full File Change Notification" );
+    items += item;
+
+    item.setIdentifier( 2 );
+    item.setLabel( "File Change Notification (EFmsisdn,EFecc,EFfdn)" );
+    items += item;
+
+    item.setIdentifier( 3 );
+    item.setLabel( "NAA Initialization+File Change Notification" );
+    items += item;
+
+    item.setIdentifier( 4 );
+    item.setLabel( "NAA Initialization" );
+    items += item;
+
+    item.setIdentifier( 5 );
+    item.setLabel( "UICC Reset" );
+    items += item;
+
+    item.setIdentifier( 6 );
+    item.setLabel( "NAA Application Reset" );
+    items += item;
+
+    item.setIdentifier( 7 );
+    item.setLabel( "NAA Session Reset" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(refreshMenuResp(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::refreshMenuResp( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() != QSimTerminalResponse::Success ) {
+        /* Unknown response - just go back to the main menu. */
+        endSession();
+
+        return;
+    }
+
+    /* Item selected. */
+
+    cmd.setType( QSimCommand::Refresh );
+    cmd.setQualifier( resp.menuItem() - 1 );
+    cmd.setDestinationDevice( QSimCommand::ME );
+    cmd.setText( "" );
+
+    if ( cmd.refreshType() == QSimCommand::FileChange ||
+            cmd.refreshType() == QSimCommand::InitAndFileChange ||
+            cmd.refreshType() == QSimCommand::NaaSessionReset ) {
+        QByteArray files;
+        files += (char) 0x03;
+        /* EFmsisdn */
+        files += (char) 0x3f;
+        files += (char) 0x00;
+        files += (char) 0x7f;
+        files += (char) 0xff;
+        files += (char) 0x6f;
+        files += (char) 0x40;
+        /* EFecc */
+        files += (char) 0x3f;
+        files += (char) 0x00;
+        files += (char) 0x7f;
+        files += (char) 0xff;
+        files += (char) 0x6f;
+        files += (char) 0xb7;
+        /* EFfdn */
+        files += (char) 0x3f;
+        files += (char) 0x00;
+        files += (char) 0x7f;
+        files += (char) 0xff;
+        files += (char) 0x6f;
+        files += (char) 0x3b;
+        cmd.addExtensionField( 0x92, files );
+    }
+
+    command( cmd, this, SLOT(endSession()) );
+}
+
+void DemoSimApplication::sendLocalInfoMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "Provide Local Information" );
+
+    item.setIdentifier( LocalInfoMenu_Time );
+    item.setLabel( "Date, time and time zone" );
+    items += item;
+
+    item.setIdentifier( LocalInfoMenu_Lang );
+    item.setLabel( "Language setting" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(localInfoMenu(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::localInfoMenu( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() == QSimTerminalResponse::Success ) {
+        switch ( resp.menuItem() ) {
+
+            case LocalInfoMenu_Time:
+            {
+                cmd.setType( QSimCommand::ProvideLocalInformation );
+                cmd.setQualifier( 0x03 );
+                command( cmd, this, SLOT(sendLocalInfoMenu()) );
+            }
+            break;
+
+            case LocalInfoMenu_Lang:
+            {
+                cmd.setType( QSimCommand::ProvideLocalInformation );
+                cmd.setQualifier( 0x04 );
+                command( cmd, this, SLOT(sendLocalInfoMenu()) );
+            }
+            break;
+
+            default:
+                endSession();
+                break;
+        }
+    } else {
+        endSession();
+    }
+}
+
+void DemoSimApplication::BIPMenu( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() == QSimTerminalResponse::Success ) {
+	switch ( resp.menuItem() ) {
+
+            case BIPMenu_OpenChannel:
+            {
+                QByteArray bearerDesc;
+                QByteArray uti;
+                QByteArray destAddress;
+                QByteArray apn;
+
+                bearerDesc += 0x02;
+                bearerDesc += 0x03;
+                bearerDesc += 0x04;
+                bearerDesc += 0x03;
+                bearerDesc += 0x04;
+                bearerDesc += 0x1F;
+                bearerDesc += 0x02;
+
+                uti += 0x01;
+                uti += 0xAD;
+                uti += 0x9C;
+
+                destAddress += 0x21;
+                destAddress += 0x01;
+                destAddress += 0x01;
+                destAddress += 0x01;
+                destAddress += 0x01;
+
+                apn += 0x06;
+                apn += 0x54;
+                apn += 0x65;
+                apn += 0x73;
+                apn += 0x74;
+                apn += 0x47;
+                apn += 0x70;
+                apn += 0x02;
+                apn += 0x72;
+                apn += 0x73;
+
+                cmd.setType( QSimCommand::OpenChannel );
+                cmd.setQualifier( QSimCommand::OpenChannelImmediate );
+                cmd.setText("Open ID");
+                cmd.setBearerDesc(bearerDesc);
+                cmd.setBufferSize(1400);
+                cmd.setApn(apn);
+                cmd.setUserLogin("UserLog");
+                cmd.setUserPassword("UserPwd");
+                cmd.setUti(uti);
+                cmd.setDestAddress(destAddress);
+                command( cmd, this, SLOT(sendBIPMenu()) );
+            }
+            break;
+
+            case BIPMenu_ReceiveData:
+            {
+                cmd.setType( QSimCommand::ReceiveData );
+                cmd.setDestinationDevice( QSimCommand::Channel1 );
+                cmd.setText("Receive Data 1");
+                cmd.setDataLength(200);
+                command( cmd, this, SLOT(sendBIPMenu()) );
+            }
+            break;
+
+            case BIPMenu_SendData:
+            {
+                QByteArray data;
+
+                data += 0x01;
+                data += 0x02;
+                data += 0x03;
+                data += 0x04;
+                data += 0x05;
+
+                cmd.setType( QSimCommand::SendData );
+                cmd.setDestinationDevice( QSimCommand::Channel1 );
+                cmd.setText("Send Data 1");
+                cmd.addExtensionField( 0x36, data );
+                command( cmd, this, SLOT(sendBIPMenu()) );
+            }
+            break;
+
+            case BIPMenu_CloseChannel:
+            {
+                cmd.setType( QSimCommand::CloseChannel );
+                cmd.setDestinationDevice( QSimCommand::Channel1 );
+                cmd.setText("Close ID 1");
+                command( cmd, this, SLOT(sendBIPMenu()) );
+            }
+            break;
+
+            case BIPMenu_GetChannelStatus:
+            {
+                cmd.setType( QSimCommand::GetChannelStatus );
+                command( cmd, this, SLOT(sendBIPMenu()) );
+            }
+            break;
+
+            default:
+                endSession();
+                break;
+        }
+    } else {
+        endSession();
+    }
+}
+
+void DemoSimApplication::sendBIPMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "BIP commands Menu" );
+
+    item.setIdentifier( BIPMenu_OpenChannel );
+    item.setLabel( "Open channel" );
+    items += item;
+
+    item.setIdentifier( BIPMenu_CloseChannel );
+    item.setLabel( "Close channel" );
+    items += item;
+
+    item.setIdentifier( BIPMenu_ReceiveData );
+    item.setLabel( "Receive data" );
+    items += item;
+
+    item.setIdentifier( BIPMenu_SendData );
+    item.setLabel( "Send data" );
+    items += item;
+
+    item.setIdentifier( BIPMenu_GetChannelStatus );
+    item.setLabel( "Get channel status" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(BIPMenu(QSimTerminalResponse)) );
+}
